﻿//!<#@ template language="C#" #>
//!<#@ samplevars Processor="DirectiveProcessor" #>
//!<# if (Params.Hosting != HostingEnvironment.NetCore) return string.Empty; #>
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;

namespace ITHit.WebDav.Server.SampleModel.__NetCore__
{
    public class Startup
    {
        public Startup(IHostingEnvironment env)
        {
            var builder = new ConfigurationBuilder()
                .SetBasePath(env.ContentRootPath)
                .AddJsonFile("appsettings.json", optional: true, reloadOnChange: true)
                .AddJsonFile("appsettings.webdav.json", optional: true, reloadOnChange: true)
                .AddJsonFile($"appsettings.{env.EnvironmentName}.json", optional: true);
            /*
            if (env.IsDevelopment())
            {
                // For more details on using the user secret store see http://go.microsoft.com/fwlink/?LinkID=532709
                builder.AddUserSecrets();
            }
            */
            HostingEnvironment = env;
            builder.AddEnvironmentVariables();
            Configuration = builder.Build();
        }

        public IConfigurationRoot Configuration { get; }
        public IHostingEnvironment HostingEnvironment { get; }

        // This method gets called by the runtime. Use this method to add services to the container.
        // For more information on how to configure your application, visit http://go.microsoft.com/fwlink/?LinkID=398940
        public void ConfigureServices(IServiceCollection services)
        {
            services.AddWebDav(Configuration, HostingEnvironment);
            //!<# if(Params.GSuite) { #>
            services.AddGSuite(Configuration, HostingEnvironment);
            //!<# } #>
            //!<# if(Params.WebSockets) { #>
            services.AddSingleton<WebSocketsService>();
            //!<# } #>      
        }

        // This method gets called by the runtime. Use this method to configure the HTTP request pipeline.
        public void Configure(IApplicationBuilder app, IHostingEnvironment env, ILoggerFactory loggerFactory)
        {
            loggerFactory.AddConsole(Configuration.GetSection("Logging"));
            loggerFactory.AddDebug();

            app.UseStaticFiles();

            if (env.IsDevelopment())
                app.UseDeveloperExceptionPage();

            //!<# if(Params.AuthenticationType.HasFlag(AuthenticationType.Basic) && Params.BuildType != BuildType.Server) { #>
            // Basic auth requires SSL connection. To enable non - SSL connection for testing purposes read the following articles:
            // - In case of Windows & MS Office: http://support.microsoft.com/kb/2123563
            // - In case of Mac OS X & MS Office: https://support.microsoft.com/en-us/kb/2498069
            app.UseBasicAuth();
            //!<# } #>
            //!<# if(Params.AuthenticationType.HasFlag(AuthenticationType.Digest) && Params.BuildType != BuildType.Server) { #>
            app.UseDigestAuth();
            //!<# } #>
            //!<# if(Params.BuildType == BuildType.Server) { #>
            // Basic auth requires SSL connection. To enable non - SSL connection for testing purposes read the following articles:
            // - In case of Windows & MS Office: http://support.microsoft.com/kb/2123563
            // - In case of Mac OS X & MS Office: https://support.microsoft.com/en-us/kb/2498069
            //app.UseBasicAuth();
            //app.UseDigestAuth();
            //!<# } #>

            //!<# if(Params.CalDav || Params.CardDav) { #>
            app.UseProvisioninge();
            //!<# } #>

            //!<# if(Params.WebSockets) { #>
            app.UseWebSockets();
            app.UseWebSocketsMiddleware();
            //!<# } #>

            //!<# if(Params.GSuite) { #>
            app.UseGSuite(HostingEnvironment);
            //!<# } #>
            app.UseWebDav(HostingEnvironment);     
        }
    }
}
